#ifndef __MERLIN_HW_H__
#define __MERLIN_HW_H__ 1

#include <stdint.h>
#include <stdbool.h>

#include <usb/LUFA/Common/Architectures.h>
#include <keyboard_settings.h>

void merlin_global_init(void);

void merlin_capture_init(void);

void merlin_main_loop(void);

#define TIMEOUT_MS                  1000
#define SIGNAL_TIMEOUT_MS           10


typedef struct x_ir_reader_t
{
	volatile bool signal;
	volatile bool log;
	volatile bool keyboard_changed;
	
	uint16_t data[X_MAX_DATA_LENGTH];
	uint8_t data_pos;

	volatile uint8_t key_ignore_time;
} x_ir_reader_t;

extern x_ir_reader_t x_ir_reader;

#if (ARCH == ARCH_AVR8)

#define CLKS_PER_ITERATION          (F_CPU/120000)

#define MERLIN_TIMER_SETUP()        TCCR1B |=  (1 << CS11)
#define MERLIN_TIMER_IRQ_ENABLE()   TIMSK1 |=  (1 << TOIE1)
#define MERLIN_TIMER_IRQ_DISABLE()  TIMSK1 &= ~(1 << TOIE1)
#define MERLIN_TIMER_IRQ_CLR()      TIFR1   =  (1 << TOV1)
#define MERLIN_TIMER_GETRES(res)    res     =  (TCNT1)      //TODO: timer auslesen geht schoener aber funktioniert ganz auch so ganz gut

#define MERLIN_EXT_IRQ_SETUP()      EICRA  |=  (1 << ISC10)
#define MERLIN_EXT_IRQ_ENABLE()     EIMSK  |=  (1 << INT1)
#define MERLIN_EXT_IRQ_DISABLE()    EIMSK  &= ~(1 << INT1)
#define MERLIN_EXT_IRQ_CLR()        EIFR    =  (1 << INTF0)
#define MERLIN_EXT_IRQ_VECT         INT1_vect

#define MERLIN_LEDS_SETUP()         DDRC = (1 << PC0) | (1 << PC1)
#define MERLIN_LED1_TGL()           PORTC ^= (1 << PC0)
#define MERLIN_LED2_TGL()           PORTC ^= (1 << PC1)

#define MERLIN_ENABLE_IRQ_GLOBAL()  sei()




#elif (ARCH == ARCH_XMEGA)

#include <config/xmega_hwconfig.h>

#define CLKS_PER_ITERATION          (F_CPU/20000 + 2400)

#define MERLIN_TIMER_SETUP()        IR_TIMER.CTRLB     = TC_WGMODE_NORMAL_gc; IR_TIMER.CTRLA = TC_CLKSEL_DIV8_gc; IR_TIMER.PER = ~0
#define MERLIN_TIMER_IRQ_ENABLE()   IR_TIMER.INTCTRLA  = TC_OVFINTLVL_MED_gc
#define MERLIN_TIMER_IRQ_DISABLE()  IR_TIMER.INTCTRLA  = TC_OVFINTLVL_OFF_gc
#define MERLIN_TIMER_IRQ_CLR()      IR_TIMER.INTFLAGS  = TC0_OVFIF_bm
#define MERLIN_TIMER_COUNT          IR_TIMER.CNT
#define MERLIN_TIMER_GETRES(res)    res                = MERLIN_TIMER_COUNT      //TODO: timer auslesen geht schoener aber funktioniert ganz auch so ganz gut
// 
#define MERLIN_EXT_IRQ_SETUP()      IR_SENS_PORT.INT0MASK = PIN0_bm; IR_SENS_PORT.PIN0CTRL = PORT_ISC_BOTHEDGES_gc
#define MERLIN_EXT_IRQ_ENABLE()     IR_SENS_PORT.INTCTRL  = PORT_INT0LVL_HI_gc
#define MERLIN_EXT_IRQ_DISABLE()    IR_SENS_PORT.INTCTRL  = PORT_INT0LVL_OFF_gc
#define MERLIN_EXT_IRQ_CLR()        IR_SENS_PORT.INTFLAGS = VPORT_INT0IF_bm

#define MERLIN_EXT_IRQ_VECT         IR_SENS_PORT_vect

#define MERLIN_ENABLE_IRQ_GLOBAL()  PMIC.CTRL = PMIC_LOLVLEN_bm | PMIC_MEDLVLEN_bm | PMIC_HILVLEN_bm; sei()

#define MERLIN_LEDS_SETUP()         DEBUG_LED_PORT.DIRSET = DEBUG_LED2 | DEBUG_LED1
#define MERLIN_LED1_TGL()           DEBUG_LED_PORT.OUTTGL = DEBUG_LED1
#define MERLIN_LED2_TGL()           DEBUG_LED_PORT.OUTTGL = DEBUG_LED2

#endif





#define TIMEOUT_ITERATIONS          ((F_CPU/1000 * TIMEOUT_MS)/CLKS_PER_ITERATION)

#define SIGNAL_TIMEOUT_ITERATIONS   ((F_CPU/1000 * SIGNAL_TIMEOUT_MS)/CLKS_PER_ITERATION)










#endif