#include "../decoder.h"


#include <stdio.h>
#include <string.h>
#include <stddef.h>
#include <stdlib.h>
#include <core/x_scancodes.h>
#include <avr/pgmspace.h>


#include <core/core.h>
#include <core/x_helpers.h>

#include <core/macros/debug.h>

#include <avr/eeprom.h>

//Timings
#define MERLIN_PERIODES_PER_SECOND    4547

//If we have n periodes per second we habe F_CPU/n clocks per periode
#define MERLIN_DEFAULT_PERIOD         (F_CPU/MERLIN_PERIODES_PER_SECOND) //In cpu clks
#define MERLIN_MAX_LEVEL              ((MERLIN_DEFAULT_PERIOD * 2.5f)/X_TIMER_DIV) //In timer clks
#define MERLIN_MAX_SHORT_LEVEL        ((MERLIN_DEFAULT_PERIOD * 1.5f)/X_TIMER_DIV)
#define MERLIN_SKIP_WIDTH             ((MERLIN_DEFAULT_PERIOD * 13.5f)/X_TIMER_DIV)

//TODO: Timings are not dynamicaly calced

#ifndef F_USB
#include <avr/iox128a1u.h>
#endif

extern const uint8_t merlin_keycodes[128];

volatile uint8_t timeout = 0;

int8_t signal_timeout(uint8_t *result)
{
// 	printf_debug("to\n");
	if(timeout < 24)
	{
		timeout++;
		if(timeout == 24)
		{
			return 0;
		}
	}
	return -1;
}

uint8_t remote_sc_2_x_sc(uint8_t rm_code)
{
	if(rm_code > 100) //Trade keycodes bigger than 100 special for saving flashmem
	{
		switch(rm_code)
		{
			case 224: //CTRL RIGHT
				return X_SC_LEFT_CONTROL;
			case 225: //SHIFT LEFT
				return X_SC_LEFT_SHIFT;
			case 226: //ALT
				return X_SC_LEFT_ALT;
			case 230: //ALTGR
				return X_SC_RIGHT_ALT;
			default:
				return X_NOT_MAPPED;
		}
	}
	
	//Else just read in the table
	return pgm_read_byte(merlin_keycodes + rm_code);
}

// void merlin_translate(uint8_t *keys, uint8_t n_keys, uint8_t *mods)

int8_t decode(uint8_t *result)
{
	uint8_t i, j;
	uint32_t time = 0;
	uint16_t p;
	uint8_t not_level = 1;
	uint16_t header = 0;
	uint8_t odd = 0;
	uint8_t count = 0;
	uint8_t use_data_bit_count = 0;
	uint8_t use_data_byte_count = 0;
	//Max 4 codes + toggle bit
	uint8_t codes[5];

// 	printf_debug("p: %d, %d\n", X_GET_NEXT_PULSE(0), );
	
	//Preamble 10000|00001
	for(i = 0; i < x_ir_reader.data_pos - 1; i++)
	{
		//Das Singal wird als Manchestercode betrachtet. Das bedeutet Folgendes:
		//Alle ungeraden Positionen sind Flanken.
		//Eine steigende Flanke -> 1
		//Eine fallende Flanke  -> 0
		
		p = X_GET_NEXT_PULSE(i);

		//################################### Error case #####################################
		if((!odd && p >= MERLIN_MAX_SHORT_LEVEL) || (p >= MERLIN_MAX_LEVEL))
		{
			//Fehlerhaft, wenn das Signal an einer geraden Position fuer 2 Pulslängen auf dem gleichen Level bleibt -> Keine Flanke an der ungeraden Position -> Es kann kein Bit gelesen werden.
			//Oder Fehlerhaft, weil das Signal fuer ungefaehr 3 oder mehr Pulslängen ohne Flanke ist. Das ist im Manchestercode unmöglich.
			
			//Da einige Infrarot-Sensoren eine Weile brauchen, um sich zu kalibrieren, akzeptieren wir es , falls die ersten Bits des Headers defekt sind.
			//Wenn weniger als 3 Bits gelesen wurden, ueberspringe die ersten 7 Bits, also 13,5 Perioden und mache bei der nächsten Flanke weiter.
			if(count >= 3)
			{
				return -1;
			}
			

			uint32_t time2 = 0;
			bool found = false;

			not_level = 1;
			//Wir ueberspringen die ersten 14 Pulse und lesen von dort weiter. So hatte der Sensor Zeit, sich zu kalibrieren.
			for(j = 0; j < x_ir_reader.data_pos - 1; j++)
			{
				time2 += (uint16_t)(x_ir_reader.data[j + 1] - x_ir_reader.data[j]);
				not_level = ~not_level & 0b1;
				if(time2 > MERLIN_SKIP_WIDTH)
				{
					header = 0b1000000;
					count = 7;
					found = true;
					break;
				}
			}
			
			if(found)
			{
				i = j;
				odd = 0;
				if(not_level)
				{
					return -1;
				}
				continue;
			}
			//Header konnte nicht gefunden werden
			return -1;

		}
		//################################# Error case done ##################################


		//Wenn die Zeitperiode < als 1,5 Perioden ist, gehen wir von *einer* Pulsbreite aus, und wir muessen "ungerade" (odd) togglen.
		if(p < MERLIN_MAX_SHORT_LEVEL)
		{
			odd = !odd;
		}
		
		//Wenn wir ungerade sind, muss das Bit gespeichert werden.
		//Dabei interessiert uns das aktuelle Signallevel, weil dadurch klar wird, ob es sich
		//um eine steigende oder fallende Flanke handelt.
		if(odd)
		{
			if(count < 10) //Die ersten 10 Bits werden im Header gespeichert
			{
				header <<= 1;
				header |= not_level;
			}
			else //Den Rest legen wir byte-weise ab.
			{    //Jedes Byte wird ein Keycode oder Modifier beinhalten
				codes[use_data_byte_count] <<= 1;
				codes[use_data_byte_count] |= not_level;
				use_data_bit_count++;
				if(use_data_bit_count == 8)
				{ //Bei Bit 8 springen wir zum nächsten Byte
					use_data_bit_count = 0;
					use_data_byte_count++;
				}
			}
			
			count++;
		}
		
		not_level = ~not_level & 0b1;

		//Wird nicht genutzt, könnte aber in Zukunft für eine Kalibrierung benutzt werden.
		time += p;
	}
	//################################# Decoden done #######################################
	
	//################################## Error Check #######################################
	//Ist der Header fehlerhaft, ignorieren wir die Daten.
	if(header != 0b1000000001)
	{
		printf_debug("header failed\n");
		return -1;
	}
	
	//Das letzte reinkommenden Bit ist invers zum Vorletzten, ansonsten hat es keine Bedeutung.
	//Ist das nicht der Fall, gehen wir von Fehlern aus.
	//Die Stopmaske besteht allerdings nur aus dem Header ohne Togglebit
	if(use_data_byte_count > 0 && codes[use_data_byte_count] == (codes[use_data_byte_count - 1] & 0b1))
	{
		return -1;
	}
	//################################# Error Check done ######################################
	

	//Translate
	for(i = 0; i < use_data_byte_count; i++)
	{
		result[i] = remote_sc_2_x_sc(codes[i]);
		if(result[i] == X_NOT_MAPPED)
		{
			printf_debug("not mapped %d\n\r", codes[i]);
			return -1;
		}
		printf_debug("rc: %d -> %d\n\r", codes[i], result[i]);
	}
	
	return use_data_byte_count;
}




//Keycode Map
const uint8_t merlin_keycodes[128] PROGMEM =
{
	X_NOT_MAPPED, /*0*/
	X_NOT_MAPPED, /*1*/
	X_NOT_MAPPED, /*2*/
	X_NOT_MAPPED, /*3*/
	X_SC_A, /*4 a*/
	X_SC_B, /*5 b*/
	X_SC_C, /*6 c*/
	X_SC_D, /*7 d*/
	X_SC_E, /*8 e*/
	X_SC_F, /*9 f*/
	X_SC_G, /*10 g*/
	X_SC_H, /*11 h*/
	X_SC_I, /*12 i*/
	X_SC_J, /*13 j*/
	X_SC_K, /*14 k*/
	X_SC_L, /*15 l*/
	X_SC_M, /*16 m*/
	X_SC_N, /*17*/
	X_SC_O, /*18*/
	X_SC_P, /*19*/
	X_SC_Q, /*20*/
	X_SC_R, /*21*/
	X_SC_S, /*22*/
	X_SC_T, /*23*/
	X_SC_U, /*24*/
	X_SC_V, /*25*/
	X_SC_W, /*26*/
	X_SC_X, /*27*/
	X_SC_Y, /*28*/
	X_SC_Z, /*29*/
	X_SC_1_AND_EXCLAMATION, /*30*/
	X_SC_2_AND_AT, /*31*/
	X_SC_3_AND_HASHMARK, /*32*/
	X_SC_4_AND_DOLLAR, /*33*/
	X_SC_5_AND_PERCENTAGE, /*34*/
	X_SC_6_AND_CARET, /*35*/
	X_SC_7_AND_AMPERSAND, /*36*/
	X_SC_8_AND_ASTERISK, /*37*/
	X_SC_9_AND_OPENING_PARENTHESIS, /*38*/
	X_SC_0_AND_CLOSING_PARENTHESIS, /*39*/
	X_SC_ENTER, /*40*/
	X_SC_POWER, /*41*/
	X_SC_BACKSPACE, /*42*/
	X_SC_TAB, /*43*/
	X_SC_SPACE, /*44*/
	X_NOT_MAPPED, /*45*/
	X_SC_EQUAL_AND_PLUS, /*46*/
	X_NOT_MAPPED, /*47*/
	X_NOT_MAPPED, /*48*/
	X_NOT_MAPPED, /*49*/
	X_NOT_MAPPED, /*50*/
	X_SC_SEMICOLON_AND_COLON, /*51*/
	X_SC_APOSTROPHE_AND_QUOTE, /*52*/
	X_NOT_MAPPED, /*53*/
	X_SC_COMMA_AND_LESS_THAN_SIGN, /*54*/
	X_SC_DOT_AND_GREATER_THAN_SIGN, /*55*/
	X_SC_SLASH_AND_QUESTION_MARK, /*56*/
	X_NOT_MAPPED, /*57*/
	X_NOT_MAPPED, /*58*/
	X_SC_SPECIAL1, /*59*/
	X_NOT_MAPPED, /*60*/
	X_SC_MEDIA_BACKWARD, /*61*/
	X_SC_MEDIA_PLAY, /*62*/
	X_SC_MEDIA_FORWARD, /*63*/
	X_SC_MEDIA_STOP, /*64*/
	X_SC_MEDIA_VOLUME_UP, /*65*/
	X_SC_MEDIA_VOLUME_DOWN, /*66*/
	X_SC_SPECIAL2, /*67*/
	X_SC_SPECIAL3, /*68*/
	X_NOT_MAPPED, /*69*/
	X_SC_LEFT_CONTROL, /*70*/
	X_SC_LEFT_SHIFT, /*71*/
	X_SC_LEFT_ALT, /*72*/
	X_SC_RIGHT_ALT, /*73*/
	X_NOT_MAPPED, /*74*/
	X_NOT_MAPPED, /*75*/
	X_NOT_MAPPED, /*76*/
	X_NOT_MAPPED, /*77*/
	X_NOT_MAPPED, /*78*/
	X_SC_RIGHT_ARROW, /*79*/
	X_SC_LEFT_ARROW, /*80*/
	X_SC_DOWN_ARROW, /*81*/
	X_SC_UP_ARROW, /*82*/
	X_NOT_MAPPED, /*83*/
	X_MACRO_SELECT, /*84*/
	X_SC_ESCAPE, /*85*/
	X_SC_SPECIAL4, /*86*/
	X_SC_SPECIAL5, /*87*/
	X_SC_SPECIAL6, /*88*/
	X_NOT_MAPPED, /*89*/
	X_NOT_MAPPED, /*90*/
	X_SC_PAGE_DOWN, /*91*/
	X_SC_PAGE_UP, /*92*/
	X_SC_MEDIA_MUTE, /*93*/
	X_NOT_MAPPED, /*94*/
	X_SC_SPECIAL7, /*95*/
	X_SC_MINUS_AND_UNDERSCORE, /*96*/ //HID_KEYBOARD_MODIFIER_LEFTCTRL
	X_SC_F2, /*97*/ //HID_KEYBOARD_MODIFIER_LEFTSHIFT
	X_SC_F5, /*98*/ //HID_KEYBOARD_MODIFIER_LEFTALT
	X_NOT_MAPPED, /*99*/
	X_SC_SPECIAL8, /*100*/
};
