#include "usb.h"
#include <avr/pgmspace.h>

keyboard_report_t keyboard_report;

mouse_report_t mouse_report;

const char usbHidReportDescriptor[USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH] PROGMEM = {
// 	0x05, 0x01,                    // USAGE_PAGE (Generic Desktop)
// 	0x09, 0x06,                    // USAGE (Keyboard)
// 	0xa1, 0x01,                    // COLLECTION (Application)
// 	0x75, 0x01,                    //   REPORT_SIZE (1)
// 	0x95, 0x08,                    //   REPORT_COUNT (8)
// 	0x05, 0x07,                    //   USAGE_PAGE (Keyboard)(Key Codes)
// 	0x19, 0xe0,                    //   USAGE_MINIMUM (Keyboard LeftControl)(224)
// 	0x29, 0xe7,                    //   USAGE_MAXIMUM (Keyboard Right GUI)(231)
// 	0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
// 	0x25, 0x01,                    //   LOGICAL_MAXIMUM (1)
// 	0x81, 0x02,                    //   INPUT (Data,Var,Abs) ; Modifier byte
// 	0x95, 0x01,                    //   REPORT_COUNT (1)
// 	0x75, 0x08,                    //   REPORT_SIZE (8)
// 	0x81, 0x03,                    //   INPUT (Cnst,Var,Abs) ; Reserved byte
// 	0x95, 0x05,                    //   REPORT_COUNT (5)
// 	0x75, 0x01,                    //   REPORT_SIZE (1)
// 	0x05, 0x08,                    //   USAGE_PAGE (LEDs)
// 	0x19, 0x01,                    //   USAGE_MINIMUM (Num Lock)
// 	0x29, 0x05,                    //   USAGE_MAXIMUM (Kana)
// 	0x91, 0x02,                    //   OUTPUT (Data,Var,Abs) ; LED report
// 	0x95, 0x01,                    //   REPORT_COUNT (1)
// 	0x75, 0x03,                    //   REPORT_SIZE (3)
// 	0x91, 0x03,                    //   OUTPUT (Cnst,Var,Abs) ; LED report padding
// 	0x95, 0x06,                    //   REPORT_COUNT (6)
// 	0x75, 0x08,                    //   REPORT_SIZE (8)
// 	0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
// 	0x25, 0x65,                    //   LOGICAL_MAXIMUM (101)
// 	0x05, 0x07,                    //   USAGE_PAGE (Keyboard)(Key Codes)
// 	0x19, 0x00,                    //   USAGE_MINIMUM (Reserved (no event indicated))(0)
// 	0x29, 0x65,                    //   USAGE_MAXIMUM (Keyboard Application)(101)
// 	0x81, 0x00,                    //   INPUT (Data,Ary,Abs)
// 	0xc0                           // END_COLLECTION
	
// 	0x05, 0x01, /* Usage Page (Generic Desktop)             */
// 	0x09, 0x02, /* Usage (Mouse)                            */
// 	0xA1, 0x01, /* Collection (Application)                 */
// 	0x09, 0x01, /*  Usage (Pointer)                         */
// 	0xA1, 0x00, /*  Collection (Physical)                   */
// 	0x85, 0x01,  /*   Report ID  */
// 	0x05, 0x09, /*      Usage Page (Buttons)                */
// 	0x19, 0x01, /*      Usage Minimum (01)                  */
// 	0x29, 0x03, /*      Usage Maximum (03)                  */
// 	0x15, 0x00, /*      Logical Minimum (0)                 */
// 	0x25, 0x01, /*      Logical Maximum (0)                 */
// 	0x95, 0x03, /*      Report Count (3)                    */
// 	0x75, 0x01, /*      Report Size (1)                     */
// 	0x81, 0x02, /*      Input (Data, Variable, Absolute)    */
// 	0x95, 0x01, /*      Report Count (1)                    */
// 	0x75, 0x05, /*      Report Size (5)                     */
// 	0x81, 0x01, /*      Input (Constant)    ;5 bit padding  */
// 	0x05, 0x01, /*      Usage Page (Generic Desktop)        */
// 	0x09, 0x30, /*      Usage (X)                           */
// 	0x09, 0x31, /*      Usage (Y)                           */
// 	0x09, 0x36, /*      Wheel                               */
// 	0x15, 0x81, /*      Logical Minimum (-127)              */
// 	0x25, 0x7F, /*      Logical Maximum (127)               */
// 	0x75, 0x08, /*      Report Size (8)                     */
// 	0x95, 0x02, /*      Report Count (3)                    */
// 	0x81, 0x06, /*      Input (Data, Variable, Relative)    */
// 	0xC0,       /*  End Collection                          */
// 	0xC0,       /* End Collection,End Collection            */
// 	//
// 	0x09, 0x06, /*		Usage (Keyboard)                    */
// 	0xA1, 0x01, /*		Collection (Application)            */
// 	0x85, 0x02,  /*   Report ID  */
// 	0x75, 0x01,                    //   REPORT_SIZE (1)
// 	0x95, 0x08,                    //   REPORT_COUNT (8)
// 	0x05, 0x07,                    //   USAGE_PAGE (Keyboard)(Key Codes)
// 	0x19, 0xe0,                    //   USAGE_MINIMUM (Keyboard LeftControl)(224)
// 	0x29, 0xe7,                    //   USAGE_MAXIMUM (Keyboard Right GUI)(231)
// 	0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
// 	0x25, 0x01,                    //   LOGICAL_MAXIMUM (1)
// 	0x81, 0x02,                    //   INPUT (Data,Var,Abs) ; Modifier byte
// 	0x95, 0x01,                    //   REPORT_COUNT (1)
// 	0x75, 0x08,                    //   REPORT_SIZE (8)
// 	0x81, 0x03,                    //   INPUT (Cnst,Var,Abs) ; Reserved byte
// 	0x95, 0x05,                    //   REPORT_COUNT (5)
// 	0x75, 0x01,                    //   REPORT_SIZE (1)
// 	0x05, 0x08,                    //   USAGE_PAGE (LEDs)
// 	0x19, 0x01,                    //   USAGE_MINIMUM (Num Lock)
// 	0x29, 0x05,                    //   USAGE_MAXIMUM (Kana)
// 	0x91, 0x02,                    //   OUTPUT (Data,Var,Abs) ; LED report
// 	0x95, 0x01,                    //   REPORT_COUNT (1)
// 	0x75, 0x03,                    //   REPORT_SIZE (3)
// 	0x91, 0x03,                    //   OUTPUT (Cnst,Var,Abs) ; LED report padding
// 	0x95, 0x06,                    //   REPORT_COUNT (6)
// 	0x75, 0x08,                    //   REPORT_SIZE (8)
// 	0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
// 	0x25, 0xFF,                    //   LOGICAL_MAXIMUM (101)
// 	0x05, 0x07,                    //   USAGE_PAGE (Keyboard)(Key Codes)
// 	0x19, 0x00,                    //   USAGE_MINIMUM (Reserved (no event indicated))(0)
// 	0x29, 0xFF,                    //   USAGE_MAXIMUM (Keyboard Application)(101)
// 	0x81, 0x00,                    //   INPUT (Data,Ary,Abs)
// 	0xC0,       /* 		End Collection,End Collection       */
	
	0x05, 0x01,                    // 52: USAGE_PAGE (Generic Desktop)
	0x09, 0x02,                    // USAGE (Mouse)
	0xa1, 0x01,                    // COLLECTION (Application)
	0x85, 0x4d,                 //   REPORT_ID (77)
	0x09, 0x01,                    //   USAGE (Pointer)
	0xa1, 0x00,                    //   COLLECTION (Physical)
	0x05, 0x09,                    //     USAGE_PAGE (Button)
	0x19, 0x01,                    //     USAGE_MINIMUM (Button 1)
	0x29, 0x03,                    //     USAGE_MAXIMUM (Button 3)
	0x15, 0x00,                    //     LOGICAL_MINIMUM (0)
	0x25, 0x01,                    //     LOGICAL_MAXIMUM (1)
	0x95, 0x03,                    //     REPORT_COUNT (3)
	0x75, 0x01,                    //     REPORT_SIZE (1)
	0x81, 0x02,                    //     INPUT (Data,Var,Abs)
	0x95, 0x01,                    //     REPORT_COUNT (1)
	0x75, 0x05,                    //     REPORT_SIZE (5)
	0x81, 0x03,                    //     INPUT (Cnst,Var,Abs)
	0x05, 0x01,                    //     USAGE_PAGE (Generic Desktop)
	0x09, 0x30,                    //     USAGE (X)
	0x09, 0x31,                    //     USAGE (Y)
	0x09, 0x38,                    //     USAGE (WHEEL)
	0x15, 0x81,                    //     LOGICAL_MINIMUM (-127)
	0x25, 0x7f,                    //     LOGICAL_MAXIMUM (127)
	0x75, 0x08,                    //     REPORT_SIZE (8)
	0x95, 0x03,                    //     REPORT_COUNT (3)
	0x81, 0x06,                    //     INPUT (Data,Var,Rel)
	0xc0,                          //   END_COLLECTION
	0xc0,                           // END_COLLECTION
	
	0x05, 0x01,                    // 37: USAGE_PAGE (Generic Desktop)
	0x09, 0x06,                    // USAGE (Keyboard)
	0xa1, 0x01,                    // COLLECTION (Application)
	0x85, 0x02,              //   REPORT_ID (75)
	0x05, 0x07,                    //   USAGE_PAGE (Keyboard)
	0x19, 0xE0,                    //   USAGE_MINIMUM (Left Ctrl)
	0x29, 0xE7,                    //   USAGE_MAXIMUM (Right GUI)
	0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
	0x25, 0x01,                    //   LOGICAL_MAXIMUM (1)
	0x75, 0x01,                    //   REPORT_SIZE (1)
	0x95, 0x08,                    //   REPORT_COUNT (8)
	0x81, 0x02,                    //   INPUT (Data,Var,Abs)
	0x95, 0x06,                    //   REPORT_COUNT (1)
	0x75, 0x08,                    //   REPORT_SIZE (8)
	0x15, 0x00,
	0x25, 0xFF,                    //   LOGICAL_MAXIMUM (101)   
	0x19, 0x00,                    //   USAGE_MINIMUM (Reserved, No Event)
	0x29, 0xFF,                    //   USAGE_MAXIMUM (Keyboard Application)
	0x81, 0x00,                    //   INPUT (Data,Var,Abs)
	0xc0                           // END_COLLECTION
};



#define NUM_LOCK 1
#define CAPS_LOCK 2
#define SCROLL_LOCK 4

volatile static uchar LED_state = 0xff; // received from PC
static uchar idleRate; // repeat rate for keyboards


usbMsgLen_t usbFunctionSetup(uchar data[8])
{
	usbRequest_t *rq = (void *)data;
	
	if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS) {
		switch(rq->bRequest) {
			case USBRQ_HID_GET_REPORT: // send "no keys pressed" if asked here
				// wValue: ReportType (highbyte), ReportID (lowbyte)
				usbMsgPtr = (short unsigned int)(void *)&keyboard_report; // we only have this one
				keyboard_report.modifier = 0;
				keyboard_report.keycode[0] = 0;
				return sizeof(keyboard_report);
			case USBRQ_HID_SET_REPORT: // if wLength == 1, should be LED state
				return (rq->wLength.word == 1) ? USB_NO_MSG : 0;
			case USBRQ_HID_GET_IDLE: // send idle rate to PC as required by spec
				usbMsgPtr = (short unsigned int)&idleRate;
				return 1;
			case USBRQ_HID_SET_IDLE: // save idle rate as required by spec
				idleRate = rq->wValue.bytes[1];
				return 0;
		}
	}
	
	return 0; // by default don't return any data
	
}



usbMsgLen_t usbFunctionWrite(uint8_t * data, uchar len) {
	if (data[0] == LED_state)
		return 1;
	else
		LED_state = data[0];
	
	// LED state changed
	if(LED_state & CAPS_LOCK)
		PORTC |= 1 << PC0; // LED on
		else
			PORTC &= ~(1 << PC0); // LED off
			
			return 1; // Data read, not expecting more
}


